@@ -1,63 +1,30 @@
 """A library of helper functions for the CherryPy test suite."""
 
 import datetime
+import io
 import logging
-log = logging.getLogger(__name__)
 import os
-thisdir = os.path.abspath(os.path.dirname(__file__))
-serverpem = os.path.join(os.getcwd(), thisdir, 'test.pem')
-import unittest
-
 import re
+import subprocess
 import sys
 import time
+import unittest
 import warnings
+import contextlib
+
+import portend
+import pytest
+
+from cheroot.test import webtest
 
 import cherrypy
-from cherrypy._cpcompat import text_or_bytes, copyitems, HTTPSConnection, ntob
+from cherrypy._cpcompat import text_or_bytes, HTTPSConnection, ntob
 from cherrypy.lib import httputil
 from cherrypy.lib import gctools
-from cherrypy.lib.reprconf import unrepr
-from cherrypy.test import webtest
 
-# Use subprocess module from Python 2.7 on Python 2.3-2.6
-if sys.version_info < (2, 7):
-    import cherrypy._cpcompat_subprocess as subprocess
-else:
-    import subprocess
-
-import nose
-
-_testconfig = None
-
-
-def get_tst_config(overconf={}):
-    global _testconfig
-    if _testconfig is None:
-        conf = {
-            'scheme': 'http',
-            'protocol': "HTTP/1.1",
-            'port': 54583,
-            'host': '127.0.0.1',
-            'validate': False,
-            'conquer': False,
-            'server': 'wsgi',
-        }
-        try:
-            import testconfig
-            _conf = testconfig.config.get('supervisor', None)
-            if _conf is not None:
-                for k, v in _conf.items():
-                    if isinstance(v, text_or_bytes):
-                        _conf[k] = unrepr(v)
-                conf.update(_conf)
-        except ImportError:
-            pass
-        _testconfig = conf
-    conf = _testconfig.copy()
-    conf.update(overconf)
-
-    return conf
+log = logging.getLogger(__name__)
+thisdir = os.path.abspath(os.path.dirname(__file__))
+serverpem = os.path.join(os.getcwd(), thisdir, 'test.pem')
 
 
 class Supervisor(object):
@@ -71,7 +38,8 @@ class Supervisor(object):
             setattr(self, k, v)
 
 
-log_to_stderr = lambda msg, level: sys.stderr.write(msg + os.linesep)
+def log_to_stderr(msg, level):
+    return sys.stderr.write(msg + os.linesep)
 
 
 class LocalSupervisor(Supervisor):
@@ -98,11 +66,10 @@ class LocalSupervisor(Supervisor):
         # place that i've found so far that I KNOW is early enough to set this.
         cherrypy.config.update({'log.screen': False})
         engine = cherrypy.engine
-        if hasattr(engine, "signal_handler"):
+        if hasattr(engine, 'signal_handler'):
             engine.signal_handler.subscribe()
-        if hasattr(engine, "console_control_handler"):
+        if hasattr(engine, 'console_control_handler'):
             engine.console_control_handler.subscribe()
-        #engine.subscribe('log', log_to_stderr)
 
     def start(self, modulename=None):
         """Load and start the HTTP server."""
@@ -126,7 +93,7 @@ class LocalSupervisor(Supervisor):
 
         cherrypy.engine.exit()
 
-        for name, server in copyitems(getattr(cherrypy, 'servers', {})):
+        for name, server in getattr(cherrypy, 'servers', {}).copy().items():
             server.unsubscribe()
             del cherrypy.servers[name]
 
@@ -135,24 +102,24 @@ class NativeServerSupervisor(LocalSupervisor):
 
     """Server supervisor for the builtin HTTP server."""
 
-    httpserver_class = "cherrypy._cpnative_server.CPHTTPServer"
+    httpserver_class = 'cherrypy._cpnative_server.CPHTTPServer'
     using_apache = False
     using_wsgi = False
 
     def __str__(self):
-        return "Builtin HTTP Server on %s:%s" % (self.host, self.port)
+        return 'Builtin HTTP Server on %s:%s' % (self.host, self.port)
 
 
 class LocalWSGISupervisor(LocalSupervisor):
 
     """Server supervisor for the builtin WSGI server."""
 
-    httpserver_class = "cherrypy._cpwsgi_server.CPWSGIServer"
+    httpserver_class = 'cherrypy._cpwsgi_server.CPWSGIServer'
     using_apache = False
     using_wsgi = True
 
     def __str__(self):
-        return "Builtin WSGI Server on %s:%s" % (self.host, self.port)
+        return 'Builtin WSGI Server on %s:%s' % (self.host, self.port)
 
     def sync_apps(self):
         """Hook a new WSGI app into the origin server."""
@@ -163,21 +130,12 @@ class LocalWSGISupervisor(LocalSupervisor):
         if app is None:
             app = cherrypy.tree
 
-        if self.conquer:
-            try:
-                import wsgiconq
-            except ImportError:
-                warnings.warn(
-                    "Error importing wsgiconq. pyconquer will not run.")
-            else:
-                app = wsgiconq.WSGILogger(app, c_calls=True)
-
         if self.validate:
             try:
                 from wsgiref import validate
             except ImportError:
                 warnings.warn(
-                    "Error importing wsgiref. The validator will not run.")
+                    'Error importing wsgiref. The validator will not run.')
             else:
                 # wraps the app in the validator
                 app = validate.validator(app)
@@ -222,8 +180,8 @@ def get_wsgi_u_supervisor(**options):
 
 class CPWebCase(webtest.WebCase):
 
-    script_name = ""
-    scheme = "http"
+    script_name = ''
+    scheme = 'http'
 
     available_servers = {'wsgi': LocalWSGISupervisor,
                          'wsgi_u': get_wsgi_u_supervisor,
@@ -234,18 +192,19 @@ class CPWebCase(webtest.WebCase):
                          'modfcgid': get_modfcgid_supervisor,
                          'modfastcgi': get_modfastcgi_supervisor,
                          }
-    default_server = "wsgi"
+    default_server = 'wsgi'
 
+    @classmethod
     def _setup_server(cls, supervisor, conf):
         v = sys.version.split()[0]
-        log.info("Python version used to run this test script: %s" % v)
-        log.info("CherryPy version: %s" % cherrypy.__version__)
-        if supervisor.scheme == "https":
-            ssl = " (ssl)"
+        log.info('Python version used to run this test script: %s' % v)
+        log.info('CherryPy version: %s' % cherrypy.__version__)
+        if supervisor.scheme == 'https':
+            ssl = ' (ssl)'
         else:
-            ssl = ""
-        log.info("HTTP server version: %s%s" % (supervisor.protocol, ssl))
-        log.info("PID: %s" % os.getpid())
+            ssl = ''
+        log.info('HTTP server version: %s%s' % (supervisor.protocol, ssl))
+        log.info('PID: %s' % os.getpid())
 
         cherrypy.server.using_apache = supervisor.using_apache
         cherrypy.server.using_wsgi = supervisor.using_wsgi
@@ -262,10 +221,10 @@ class CPWebCase(webtest.WebCase):
         baseconf.update({'server.socket_host': supervisor.host,
                          'server.socket_port': supervisor.port,
                          'server.protocol_version': supervisor.protocol,
-                         'environment': "test_suite",
+                         'environment': 'test_suite',
                          })
-        if supervisor.scheme == "https":
-            #baseconf['server.ssl_module'] = 'builtin'
+        if supervisor.scheme == 'https':
+            # baseconf['server.ssl_module'] = 'builtin'
             baseconf['server.ssl_certificate'] = serverpem
             baseconf['server.ssl_private_key'] = serverpem
 
@@ -275,15 +234,22 @@ class CPWebCase(webtest.WebCase):
         # exactly like each test module does, because a relative import
         # would stick a second instance of webtest in sys.modules,
         # and we wouldn't be able to globally override the port anymore.
-        if supervisor.scheme == "https":
+        if supervisor.scheme == 'https':
             webtest.WebCase.HTTP_CONN = HTTPSConnection
         return baseconf
-    _setup_server = classmethod(_setup_server)
 
+    @classmethod
     def setup_class(cls):
         ''
         # Creates a server
-        conf = get_tst_config()
+        conf = {
+            'scheme': 'http',
+            'protocol': 'HTTP/1.1',
+            'port': 54583,
+            'host': '127.0.0.1',
+            'validate': False,
+            'server': 'wsgi',
+        }
         supervisor_factory = cls.available_servers.get(
             conf.get('server', 'wsgi'))
         if supervisor_factory is None:
@@ -309,13 +275,12 @@ class CPWebCase(webtest.WebCase):
             supervisor.start(cls.__module__)
 
         cls.supervisor = supervisor
-    setup_class = classmethod(setup_class)
 
+    @classmethod
     def teardown_class(cls):
         ''
         if hasattr(cls, 'setup_server'):
             cls.supervisor.stop()
-    teardown_class = classmethod(teardown_class)
 
     do_gc_test = False
 
@@ -323,29 +288,29 @@ class CPWebCase(webtest.WebCase):
         if not self.do_gc_test:
             return
 
-        self.getPage("/gc/stats")
+        self.getPage('/gc/stats')
         try:
-            self.assertBody("Statistics:")
+            self.assertBody('Statistics:')
         except Exception:
-            "Failures occur intermittently. See #1420"
+            'Failures occur intermittently. See #1420'
 
     def prefix(self):
-        return self.script_name.rstrip("/")
+        return self.script_name.rstrip('/')
 
     def base(self):
-        if ((self.scheme == "http" and self.PORT == 80) or
-                (self.scheme == "https" and self.PORT == 443)):
-            port = ""
+        if ((self.scheme == 'http' and self.PORT == 80) or
+                (self.scheme == 'https' and self.PORT == 443)):
+            port = ''
         else:
-            port = ":%s" % self.PORT
+            port = ':%s' % self.PORT
 
-        return "%s://%s%s%s" % (self.scheme, self.HOST, port,
-                                self.script_name.rstrip("/"))
+        return '%s://%s%s%s' % (self.scheme, self.HOST, port,
+                                self.script_name.rstrip('/'))
 
     def exit(self):
         sys.exit()
 
-    def getPage(self, url, headers=None, method="GET", body=None,
+    def getPage(self, url, headers=None, method='GET', body=None,
                 protocol=None, raise_subcls=None):
         """Open the url. Return status, headers, body.
 
@@ -360,7 +325,7 @@ class CPWebCase(webtest.WebCase):
                                        protocol, raise_subcls)
 
     def skip(self, msg='skipped '):
-        raise nose.SkipTest(msg)
+        pytest.skip(msg)
 
     def assertErrorPage(self, status, message=None, pattern=''):
         """Compare the response body with a built in error page.
@@ -378,7 +343,7 @@ class CPWebCase(webtest.WebCase):
         epage = re.escape(page)
         epage = epage.replace(
             esc('<pre id="traceback"></pre>'),
-            esc('<pre id="traceback">') + ntob('(.*)') + esc('</pre>'))
+            esc('<pre id="traceback">') + b'(.*)' + esc('</pre>'))
         m = re.match(epage, self.body, re.DOTALL)
         if not m:
             self._handlewebError(
@@ -424,6 +389,8 @@ def _test_method_sorter(_, x, y):
     if x < y:
         return -1
     return 0
+
+
 unittest.TestLoader.sortTestMethodsUsing = _test_method_sorter
 
 
@@ -462,7 +429,7 @@ log.access_file: r'%(access_log)s'
         self.host = socket_host or cherrypy.server.socket_host
         self.port = socket_port or cherrypy.server.socket_port
 
-    def write_conf(self, extra=""):
+    def write_conf(self, extra=''):
         if self.ssl:
             serverpem = os.path.join(thisdir, 'test.pem')
             ssl = """
@@ -470,7 +437,7 @@ server.ssl_certificate: r'%s'
 server.ssl_private_key: r'%s'
 """ % (serverpem, serverpem)
         else:
-            ssl = ""
+            ssl = ''
 
         conf = self.config_template % {
             'host': self.host,
@@ -480,19 +447,37 @@ server.ssl_private_key: r'%s'
             'ssl': ssl,
             'extra': extra,
         }
-        f = open(self.config_file, 'wb')
-        f.write(ntob(conf, 'utf-8'))
-        f.close()
+        with io.open(self.config_file, 'w', encoding='utf-8') as f:
+            f.write(str(conf))
 
     def start(self, imports=None):
         """Start cherryd in a subprocess."""
-        cherrypy._cpserver.wait_for_free_port(self.host, self.port)
+        portend.free(self.host, self.port, timeout=1)
 
         args = [
-            os.path.join(thisdir, '..', 'cherryd'),
+            '-m',
+            'cherrypy',
             '-c', self.config_file,
             '-p', self.pid_file,
         ]
+        r"""
+        Command for running cherryd server with autoreload enabled
+
+        Using
+
+        ```
+        ['-c',
+         "__requires__ = 'CherryPy'; \
+         import pkg_resources, re, sys; \
+         sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0]); \
+         sys.exit(\
+            pkg_resources.load_entry_point(\
+                'CherryPy', 'console_scripts', 'cherryd')())"]
+        ```
+
+        doesn't work as it's impossible to reconstruct the `-c`'s contents.
+        Ref: https://github.com/cherrypy/cherrypy/issues/1545
+        """
 
         if not isinstance(imports, (list, tuple)):
             imports = [imports]
@@ -517,7 +502,7 @@ server.ssl_private_key: r'%s'
         if self.wait:
             self.exit_code = self._proc.wait()
         else:
-            cherrypy._cpserver.wait_for_occupied_port(self.host, self.port)
+            portend.occupied(self.host, self.port, timeout=5)
 
         # Give the engine a wee bit more time to finish STARTING
         if self.daemonize:
@@ -537,20 +522,5 @@ server.ssl_private_key: r'%s'
         self._proc.wait()
 
     def _join_daemon(self):
-        try:
-            try:
-                # Mac, UNIX
-                os.wait()
-            except AttributeError:
-                # Windows
-                try:
-                    pid = self.get_pid()
-                except IOError:
-                    # Assume the subprocess deleted the pidfile on shutdown.
-                    pass
-                else:
-                    os.waitpid(pid, 0)
-        except OSError:
-            x = sys.exc_info()[1]
-            if x.args != (10, 'No child processes'):
-                raise
+        with contextlib.suppress(IOError):
+            os.waitpid(self.get_pid(), 0)
